from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="SystemStats")


@attr.s(auto_attribs=True)
class SystemStats:
    """ """

    buddy_list_size: Union[Unset, int] = UNSET
    max_buddy_list_size: Union[Unset, int] = UNSET
    priv_channel_size: Union[Unset, int] = UNSET
    org_size: Union[Unset, int] = UNSET
    charinfo_cache_size: Union[Unset, int] = UNSET
    chatqueue_length: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        buddy_list_size = self.buddy_list_size
        max_buddy_list_size = self.max_buddy_list_size
        priv_channel_size = self.priv_channel_size
        org_size = self.org_size
        charinfo_cache_size = self.charinfo_cache_size
        chatqueue_length = self.chatqueue_length

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if buddy_list_size is not UNSET:
            field_dict["buddy_list_size"] = buddy_list_size
        if max_buddy_list_size is not UNSET:
            field_dict["max_buddy_list_size"] = max_buddy_list_size
        if priv_channel_size is not UNSET:
            field_dict["priv_channel_size"] = priv_channel_size
        if org_size is not UNSET:
            field_dict["org_size"] = org_size
        if charinfo_cache_size is not UNSET:
            field_dict["charinfo_cache_size"] = charinfo_cache_size
        if chatqueue_length is not UNSET:
            field_dict["chatqueue_length"] = chatqueue_length

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        buddy_list_size = d.pop("buddy_list_size", UNSET)

        max_buddy_list_size = d.pop("max_buddy_list_size", UNSET)

        priv_channel_size = d.pop("priv_channel_size", UNSET)

        org_size = d.pop("org_size", UNSET)

        charinfo_cache_size = d.pop("charinfo_cache_size", UNSET)

        chatqueue_length = d.pop("chatqueue_length", UNSET)

        system_stats = cls(
            buddy_list_size=buddy_list_size,
            max_buddy_list_size=max_buddy_list_size,
            priv_channel_size=priv_channel_size,
            org_size=org_size,
            charinfo_cache_size=charinfo_cache_size,
            chatqueue_length=chatqueue_length,
        )

        system_stats.additional_properties = d
        return system_stats

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="RouteHopFormat")


@attr.s(auto_attribs=True)
class RouteHopFormat:
    """ """

    hop: Union[Unset, str] = UNSET
    where: Union[Unset, None, str] = UNSET
    render: Union[Unset, bool] = UNSET
    format_: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        hop = self.hop
        where = self.where
        render = self.render
        format_ = self.format_

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if hop is not UNSET:
            field_dict["hop"] = hop
        if where is not UNSET:
            field_dict["where"] = where
        if render is not UNSET:
            field_dict["render"] = render
        if format_ is not UNSET:
            field_dict["format"] = format_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        hop = d.pop("hop", UNSET)

        where = d.pop("where", UNSET)

        render = d.pop("render", UNSET)

        format_ = d.pop("format", UNSET)

        route_hop_format = cls(
            hop=hop,
            where=where,
            render=render,
            format_=format_,
        )

        route_hop_format.additional_properties = d
        return route_hop_format

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union

import httpx

from ...client import AuthenticatedClient
from ...models.module_command import ModuleCommand
from ...models.module_subcommand_channel import ModuleSubcommandChannel
from ...types import Response


def _get_kwargs(
    module: str,
    command: str,
    channel: str,
    *,
    client: AuthenticatedClient,
    json_body: ModuleSubcommandChannel,
) -> Dict[str, Any]:
    url = "{}/module/{module}/commands/{command}/{channel}".format(
        client.base_url, module=module, command=command, channel=channel
    )

    headers: Dict[str, Any] = client.get_headers()

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, ModuleCommand]]:
    if response.status_code == 200:
        response_200 = ModuleCommand.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = None

        return response_422
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, ModuleCommand]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    module: str,
    command: str,
    channel: str,
    *,
    client: AuthenticatedClient,
    json_body: ModuleSubcommandChannel,
) -> Response[Union[Any, ModuleCommand]]:
    kwargs = _get_kwargs(
        module=module,
        command=command,
        channel=channel,
        client=client,
        json_body=json_body,
    )

    response = client.client.put(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    module: str,
    command: str,
    channel: str,
    *,
    client: AuthenticatedClient,
    json_body: ModuleSubcommandChannel,
) -> Optional[Union[Any, ModuleCommand]]:
    """Activate or deactivate a Command"""

    return sync_detailed(
        module=module,
        command=command,
        channel=channel,
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    module: str,
    command: str,
    channel: str,
    *,
    client: AuthenticatedClient,
    json_body: ModuleSubcommandChannel,
) -> Response[Union[Any, ModuleCommand]]:
    kwargs = _get_kwargs(
        module=module,
        command=command,
        channel=channel,
        client=client,
        json_body=json_body,
    )

    response = await client.async_client.put(**kwargs)

    return _build_response(response=response)


async def asyncio(
    module: str,
    command: str,
    channel: str,
    *,
    client: AuthenticatedClient,
    json_body: ModuleSubcommandChannel,
) -> Optional[Union[Any, ModuleCommand]]:
    """Activate or deactivate a Command"""

    return (
        await asyncio_detailed(
            module=module,
            command=command,
            channel=channel,
            client=client,
            json_body=json_body,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ConfigModule")


@attr.s(auto_attribs=True)
class ConfigModule:
    """ """

    name: Union[Unset, str] = UNSET
    num_commands_enabled: Union[Unset, int] = UNSET
    num_commands_disabled: Union[Unset, int] = UNSET
    num_events_enabled: Union[Unset, int] = UNSET
    num_events_disabled: Union[Unset, int] = UNSET
    num_settings: Union[Unset, int] = UNSET
    description: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        num_commands_enabled = self.num_commands_enabled
        num_commands_disabled = self.num_commands_disabled
        num_events_enabled = self.num_events_enabled
        num_events_disabled = self.num_events_disabled
        num_settings = self.num_settings
        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if num_commands_enabled is not UNSET:
            field_dict["num_commands_enabled"] = num_commands_enabled
        if num_commands_disabled is not UNSET:
            field_dict["num_commands_disabled"] = num_commands_disabled
        if num_events_enabled is not UNSET:
            field_dict["num_events_enabled"] = num_events_enabled
        if num_events_disabled is not UNSET:
            field_dict["num_events_disabled"] = num_events_disabled
        if num_settings is not UNSET:
            field_dict["num_settings"] = num_settings
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        num_commands_enabled = d.pop("num_commands_enabled", UNSET)

        num_commands_disabled = d.pop("num_commands_disabled", UNSET)

        num_events_enabled = d.pop("num_events_enabled", UNSET)

        num_events_disabled = d.pop("num_events_disabled", UNSET)

        num_settings = d.pop("num_settings", UNSET)

        description = d.pop("description", UNSET)

        config_module = cls(
            name=name,
            num_commands_enabled=num_commands_enabled,
            num_commands_disabled=num_commands_disabled,
            num_events_enabled=num_events_enabled,
            num_events_disabled=num_events_disabled,
            num_settings=num_settings,
            description=description,
        )

        config_module.additional_properties = d
        return config_module

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.proxy_capabilities import ProxyCapabilities
from ..types import UNSET, Unset

T = TypeVar("T", bound="MiscSystemInformation")


@attr.s(auto_attribs=True)
class MiscSystemInformation:
    """ """

    using_chat_proxy: Union[Unset, bool] = UNSET
    proxy_capabilities: Union[Unset, ProxyCapabilities] = UNSET
    uptime: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        using_chat_proxy = self.using_chat_proxy
        proxy_capabilities: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.proxy_capabilities, Unset):
            proxy_capabilities = self.proxy_capabilities.to_dict()

        uptime = self.uptime

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if using_chat_proxy is not UNSET:
            field_dict["using_chat_proxy"] = using_chat_proxy
        if proxy_capabilities is not UNSET:
            field_dict["proxy_capabilities"] = proxy_capabilities
        if uptime is not UNSET:
            field_dict["uptime"] = uptime

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        using_chat_proxy = d.pop("using_chat_proxy", UNSET)

        _proxy_capabilities = d.pop("proxy_capabilities", UNSET)
        proxy_capabilities: Union[Unset, ProxyCapabilities]
        if isinstance(_proxy_capabilities, Unset):
            proxy_capabilities = UNSET
        else:
            proxy_capabilities = ProxyCapabilities.from_dict(_proxy_capabilities)

        uptime = d.pop("uptime", UNSET)

        misc_system_information = cls(
            using_chat_proxy=using_chat_proxy,
            proxy_capabilities=proxy_capabilities,
            uptime=uptime,
        )

        misc_system_information.additional_properties = d
        return misc_system_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.relay_layer import RelayLayer
from ..types import UNSET, Unset

T = TypeVar("T", bound="RelayConfig")


@attr.s(auto_attribs=True)
class RelayConfig:
    """ """

    name: Union[Unset, str] = UNSET
    layers: Union[Unset, List[RelayLayer]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        layers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.layers, Unset):
            layers = []
            for layers_item_data in self.layers:
                layers_item = layers_item_data.to_dict()

                layers.append(layers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if layers is not UNSET:
            field_dict["layers"] = layers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        layers = []
        _layers = d.pop("layers", UNSET)
        for layers_item_data in _layers or []:
            layers_item = RelayLayer.from_dict(layers_item_data)

            layers.append(layers_item)

        relay_config = cls(
            name=name,
            layers=layers,
        )

        relay_config.additional_properties = d
        return relay_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.setting_option import SettingOption
from ..types import UNSET, Unset

T = TypeVar("T", bound="ModuleSetting")


@attr.s(auto_attribs=True)
class ModuleSetting:
    """ """

    type: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    value: Union[None, Unset, bool, int, str] = UNSET
    options: Union[Unset, List[SettingOption]] = UNSET
    editable: Union[Unset, bool] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        name = self.name
        value: Union[None, Unset, bool, int, str]
        if isinstance(self.value, Unset):
            value = UNSET
        elif self.value is None:
            value = None
        else:
            value = self.value

        options: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.options, Unset):
            options = []
            for options_item_data in self.options:
                options_item = options_item_data.to_dict()

                options.append(options_item)

        editable = self.editable
        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if name is not UNSET:
            field_dict["name"] = name
        if value is not UNSET:
            field_dict["value"] = value
        if options is not UNSET:
            field_dict["options"] = options
        if editable is not UNSET:
            field_dict["editable"] = editable
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        name = d.pop("name", UNSET)

        def _parse_value(data: object) -> Union[None, Unset, bool, int, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, bool, int, str], data)

        value = _parse_value(d.pop("value", UNSET))

        options = []
        _options = d.pop("options", UNSET)
        for options_item_data in _options or []:
            options_item = SettingOption.from_dict(options_item_data)

            options.append(options_item)

        editable = d.pop("editable", UNSET)

        description = d.pop("description", UNSET)

        module_setting = cls(
            type=type,
            name=name,
            value=value,
            options=options,
            editable=editable,
            description=description,
        )

        module_setting.additional_properties = d
        return module_setting

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.function_parameter import FunctionParameter
from ..types import UNSET, Unset

T = TypeVar("T", bound="ClassSpec")


@attr.s(auto_attribs=True)
class ClassSpec:
    """ """

    name: Union[Unset, str] = UNSET
    class_: Union[Unset, str] = UNSET
    params: Union[Unset, List[FunctionParameter]] = UNSET
    description: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        class_ = self.class_
        params: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.params, Unset):
            params = []
            for params_item_data in self.params:
                params_item = params_item_data.to_dict()

                params.append(params_item)

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if class_ is not UNSET:
            field_dict["class"] = class_
        if params is not UNSET:
            field_dict["params"] = params
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        class_ = d.pop("class", UNSET)

        params = []
        _params = d.pop("params", UNSET)
        for params_item_data in _params or []:
            params_item = FunctionParameter.from_dict(params_item_data)

            params.append(params_item)

        description = d.pop("description", UNSET)

        class_spec = cls(
            name=name,
            class_=class_,
            params=params,
            description=description,
        )

        class_spec.additional_properties = d
        return class_spec

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.relay_layer_argument import RelayLayerArgument
from ..types import UNSET, Unset

T = TypeVar("T", bound="RelayLayer")


@attr.s(auto_attribs=True)
class RelayLayer:
    """ """

    layer: Union[Unset, str] = UNSET
    arguments: Union[Unset, List[RelayLayerArgument]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        layer = self.layer
        arguments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.arguments, Unset):
            arguments = []
            for arguments_item_data in self.arguments:
                arguments_item = arguments_item_data.to_dict()

                arguments.append(arguments_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if layer is not UNSET:
            field_dict["layer"] = layer
        if arguments is not UNSET:
            field_dict["arguments"] = arguments

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        layer = d.pop("layer", UNSET)

        arguments = []
        _arguments = d.pop("arguments", UNSET)
        for arguments_item_data in _arguments or []:
            arguments_item = RelayLayerArgument.from_dict(arguments_item_data)

            arguments.append(arguments_item)

        relay_layer = cls(
            layer=layer,
            arguments=arguments,
        )

        relay_layer.additional_properties = d
        return relay_layer

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ProxyCapabilities")


@attr.s(auto_attribs=True)
class ProxyCapabilities:
    """ """

    type: Union[Unset, str] = UNSET
    name: Union[Unset, None, str] = UNSET
    version: Union[Unset, None, str] = UNSET
    send_modes: Union[Unset, List[str]] = UNSET
    buddy_modes: Union[Unset, List[str]] = UNSET
    supported_cmds: Union[Unset, List[str]] = UNSET
    rate_limited: Union[Unset, bool] = UNSET
    default_mode: Union[Unset, None, str] = UNSET
    started_at: Union[Unset, None, int] = UNSET
    workers: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        name = self.name
        version = self.version
        send_modes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.send_modes, Unset):
            send_modes = self.send_modes

        buddy_modes: Union[Unset, List[str]] = UNSET
        if not isinstance(self.buddy_modes, Unset):
            buddy_modes = self.buddy_modes

        supported_cmds: Union[Unset, List[str]] = UNSET
        if not isinstance(self.supported_cmds, Unset):
            supported_cmds = self.supported_cmds

        rate_limited = self.rate_limited
        default_mode = self.default_mode
        started_at = self.started_at
        workers: Union[Unset, List[str]] = UNSET
        if not isinstance(self.workers, Unset):
            workers = self.workers

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if name is not UNSET:
            field_dict["name"] = name
        if version is not UNSET:
            field_dict["version"] = version
        if send_modes is not UNSET:
            field_dict["send-modes"] = send_modes
        if buddy_modes is not UNSET:
            field_dict["buddy-modes"] = buddy_modes
        if supported_cmds is not UNSET:
            field_dict["supported-cmds"] = supported_cmds
        if rate_limited is not UNSET:
            field_dict["rate-limited"] = rate_limited
        if default_mode is not UNSET:
            field_dict["default-mode"] = default_mode
        if started_at is not UNSET:
            field_dict["started-at"] = started_at
        if workers is not UNSET:
            field_dict["workers"] = workers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = d.pop("type", UNSET)

        name = d.pop("name", UNSET)

        version = d.pop("version", UNSET)

        send_modes = cast(List[str], d.pop("send-modes", UNSET))

        buddy_modes = cast(List[str], d.pop("buddy-modes", UNSET))

        supported_cmds = cast(List[str], d.pop("supported-cmds", UNSET))

        rate_limited = d.pop("rate-limited", UNSET)

        default_mode = d.pop("default-mode", UNSET)

        started_at = d.pop("started-at", UNSET)

        workers = cast(List[str], d.pop("workers", UNSET))

        proxy_capabilities = cls(
            type=type,
            name=name,
            version=version,
            send_modes=send_modes,
            buddy_modes=buddy_modes,
            supported_cmds=supported_cmds,
            rate_limited=rate_limited,
            default_mode=default_mode,
            started_at=started_at,
            workers=workers,
        )

        proxy_capabilities.additional_properties = d
        return proxy_capabilities

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="MemoryInformation")


@attr.s(auto_attribs=True)
class MemoryInformation:
    """ """

    current_usage: Union[Unset, int] = UNSET
    current_usage_real: Union[Unset, int] = UNSET
    peak_usage: Union[Unset, int] = UNSET
    peak_usage_real: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        current_usage = self.current_usage
        current_usage_real = self.current_usage_real
        peak_usage = self.peak_usage
        peak_usage_real = self.peak_usage_real

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if current_usage is not UNSET:
            field_dict["current_usage"] = current_usage
        if current_usage_real is not UNSET:
            field_dict["current_usage_real"] = current_usage_real
        if peak_usage is not UNSET:
            field_dict["peak_usage"] = peak_usage
        if peak_usage_real is not UNSET:
            field_dict["peak_usage_real"] = peak_usage_real

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        current_usage = d.pop("current_usage", UNSET)

        current_usage_real = d.pop("current_usage_real", UNSET)

        peak_usage = d.pop("peak_usage", UNSET)

        peak_usage_real = d.pop("peak_usage_real", UNSET)

        memory_information = cls(
            current_usage=current_usage,
            current_usage_real=current_usage_real,
            peak_usage=peak_usage,
            peak_usage_real=peak_usage_real,
        )

        memory_information.additional_properties = d
        return memory_information

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.online_player import OnlinePlayer
from ..types import UNSET, Unset

T = TypeVar("T", bound="OnlinePlayers")


@attr.s(auto_attribs=True)
class OnlinePlayers:
    """This is the list of all players considered to be online by the bot"""

    org: Union[Unset, List[OnlinePlayer]] = UNSET
    private_channel: Union[Unset, List[OnlinePlayer]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        org: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.org, Unset):
            org = []
            for org_item_data in self.org:
                org_item = org_item_data.to_dict()

                org.append(org_item)

        private_channel: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.private_channel, Unset):
            private_channel = []
            for private_channel_item_data in self.private_channel:
                private_channel_item = private_channel_item_data.to_dict()

                private_channel.append(private_channel_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if org is not UNSET:
            field_dict["org"] = org
        if private_channel is not UNSET:
            field_dict["private_channel"] = private_channel

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        org = []
        _org = d.pop("org", UNSET)
        for org_item_data in _org or []:
            org_item = OnlinePlayer.from_dict(org_item_data)

            org.append(org_item)

        private_channel = []
        _private_channel = d.pop("private_channel", UNSET)
        for private_channel_item_data in _private_channel or []:
            private_channel_item = OnlinePlayer.from_dict(private_channel_item_data)

            private_channel.append(private_channel_item)

        online_players = cls(
            org=org,
            private_channel=private_channel,
        )

        online_players.additional_properties = d
        return online_players

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from base64 import b64encode
from time import time_ns
from typing import Dict

import attr
import httpx
from OpenSSL.crypto import FILETYPE_PEM, load_privatekey, sign


@attr.s(auto_attribs=True)
class Client:
    """A class for keeping track of data related to the API"""

    base_url: str
    headers: Dict[str, str] = attr.ib(factory=dict, kw_only=True)
    timeout: float = attr.ib(5.0, kw_only=True)

    client: httpx.Client = attr.ib(factory=httpx.Client, init=False)
    async_client: httpx.AsyncClient = attr.ib(factory=httpx.AsyncClient, init=False)

    async def close(self) -> None:
        await self.async_client.aclose()

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in all endpoints"""
        return {**self.headers}

    def with_headers(self, headers: Dict[str, str]) -> "Client":
        """Get a new client matching this one with additional headers"""
        return attr.evolve(self, headers={**self.headers, **headers})

    def get_timeout(self) -> float:
        return self.timeout

    def with_timeout(self, timeout: float) -> "Client":
        """Get a new client matching this one with a new timeout (in seconds)"""
        return attr.evolve(self, timeout=timeout)


class AuthenticatedClient(Client):
    pass


@attr.s(auto_attribs=True)
class BasicAuthClient(AuthenticatedClient):
    """A Client which has been authenticated for use on secured endpoints"""

    username: str
    password: str

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in authenticated endpoints"""
        basic_auth = b64encode(f"{self.username}:{self.password}".encode()).decode("ascii")
        return {"Authorization": f"Basic {basic_auth}", **self.headers}


@attr.s(auto_attribs=True)
class SignedAuthClient(AuthenticatedClient):
    """A Client which has been authenticated for use on secured endpoints"""

    key_id: str
    private_key: str

    def get_headers(self) -> Dict[str, str]:
        """Get headers to be used in authenticated endpoints"""
        request_id = f"{time_ns() // 1000000}"
        key = load_privatekey(FILETYPE_PEM, self.private_key)
        signature = b64encode(sign(key, request_id.encode(), "sha512")).decode("ascii")
        header_value = f'keyId="{self.key_id}",algorithm="sha512",sequence="{request_id}",signature="{signature}"'
        return {"Signature": header_value, **self.headers}



from typing import Any, Dict

import httpx

from ...client import AuthenticatedClient
from ...types import Response


def _get_kwargs(
    uuid: str,
    *,
    client: AuthenticatedClient,
    json_body: str,
) -> Dict[str, Any]:
    url = "{}/execute/{uuid}".format(client.base_url, uuid=uuid)

    headers: Dict[str, Any] = client.get_headers()

    json_json_body = json_body

    return {
        "url": url,
        "headers": headers,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    uuid: str,
    *,
    client: AuthenticatedClient,
    json_body: str,
) -> Response[Any]:
    kwargs = _get_kwargs(
        uuid=uuid,
        client=client,
        json_body=json_body,
    )

    response = client.client.post(
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    uuid: str,
    *,
    client: AuthenticatedClient,
    json_body: str,
) -> Response[Any]:
    kwargs = _get_kwargs(
        uuid=uuid,
        client=client,
        json_body=json_body,
    )

    response = await client.async_client.post(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, Union

import httpx

from ...client import AuthenticatedClient
from ...models.operation import Operation
from ...types import UNSET, Response, Unset


def _get_kwargs(
    module: str,
    *,
    client: AuthenticatedClient,
    json_body: Operation,
    channel: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/module/{module}".format(client.base_url, module=module)

    headers: Dict[str, Any] = client.get_headers()

    params: Dict[str, Any] = {
        "channel": channel,
    }
    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "url": url,
        "headers": headers,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    module: str,
    *,
    client: AuthenticatedClient,
    json_body: Operation,
    channel: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    kwargs = _get_kwargs(
        module=module,
        client=client,
        json_body=json_body,
        channel=channel,
    )

    response = client.client.patch(
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    module: str,
    *,
    client: AuthenticatedClient,
    json_body: Operation,
    channel: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    kwargs = _get_kwargs(
        module=module,
        client=client,
        json_body=json_body,
        channel=channel,
    )

    response = await client.async_client.patch(**kwargs)

    return _build_response(response=response)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nadypy', 'nadypy.api', 'nadypy.api.default', 'nadypy.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<22.0.0',
 'httpx>=0.15.4,<0.19.0',
 'pyopenssl>=20.0.1,<21.0.0',
 'python-dateutil>=2.8.0,<3.0.0']

setup_kwargs = {
    'name': 'nadypy',
    'version': '5.2.0',
    'description': 'A client library for accessing Nadybot API',
    'long_description': '# nadypy\n\nA client library for accessing a [Nadybot](https://github.com/Nadybot/Nadybot)\'s API.\n\n## Installation\n\nFrom PyPi:\n\n```shell\npip install nadypy\n```\n\nFrom GitHub:\n\n```shell\npip install git+https://github.com/Nadybot/nadypy.git\n```\n\n## Usage\n\nFirst, create a client:\n\n```python\nfrom nadypy import Client\n\nclient = Client(base_url="http://localhost:8080/api")\n```\n\nIf the endpoints you\'re going to hit require authentication (this currently applies to **all** endpoints), use either a `BasicAuthClient` or a `SignedAuthClient` instead.\n\n`BasicAuthClient` uses credentials acquired via `!webauth`, which are not valid permanently:\n\n```python\nfrom nadypy import BasicAuthClient\n\nclient = BasicAuthClient(base_url="http://localhost:8080/api", username="Character", password="password")\n```\n\n`SignedAuthClient` uses private keys as explained [here](https://github.com/Nadybot/Nadybot/wiki/REST-API):\n\n```python\nfrom nadypy import SignedAuthClient\n\nsigned_auth_client = SignedAuthClient(\n    "http://localhost:8080/api",\n    key_id="bd879e20",\n    private_key="""\\\n-----BEGIN EC PRIVATE KEY-----\nMHcCAQEEINca+XgCZoLXuu6p77cphsIxMiSaG09tBH6SV9AgEH4ioAoGCCqGSM49\nAwEHoUQDQgAEPnzqwJq/el8kyNSPmYhQJ0L2qrMFtM3XDbAHrTQlXbFN2G8NmMBp\ni52oubVjuTSHol1BQf4Haftbt0oBvHGUIw==\n-----END EC PRIVATE KEY-----\n""",\n)\n```\n\nNow call your endpoint and use your models:\n\n```python\nfrom typing import Optional\n\nfrom nadypy.models import SystemInformation\nfrom nadypy.api.default import get_sysinfo\nfrom nadybot_api_client.types import Response\n\nsysinfo: Optional[SystemInformation] = get_sysinfo.sync(client=client)\n# or if you need more info (e.g. status_code)\nresponse: Response[SystemInformation] = get_sysinfo.sync_detailed(client=client)\n```\n\nOr do the same thing with an async version:\n\n```python\nfrom typing import Optional\n\nfrom nadypy.models import SystemInformation\nfrom nadypy.api.default import get_sysinfo\nfrom nadybot_api_client.types import Response\n\nsysinfo: Optional[SystemInformation] = await get_sysinfo.asyncio(client=client)\n# or if you need more info (e.g. status_code)\nresponse: Response[SystemInformation] = await get_sysinfo.asyncio_detailed(client=client)\n```\n',
    'author': 'Jens Reidel',
    'author_email': 'adrian@travitia.xyz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ConfigStatistics")


@attr.s(auto_attribs=True)
class ConfigStatistics:
    """ """

    active_tell_commands: Union[Unset, int] = UNSET
    active_priv_commands: Union[Unset, int] = UNSET
    active_org_commands: Union[Unset, int] = UNSET
    active_subcommands: Union[Unset, int] = UNSET
    active_aliases: Union[Unset, int] = UNSET
    active_events: Union[Unset, int] = UNSET
    active_help_commands: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        active_tell_commands = self.active_tell_commands
        active_priv_commands = self.active_priv_commands
        active_org_commands = self.active_org_commands
        active_subcommands = self.active_subcommands
        active_aliases = self.active_aliases
        active_events = self.active_events
        active_help_commands = self.active_help_commands

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if active_tell_commands is not UNSET:
            field_dict["active_tell_commands"] = active_tell_commands
        if active_priv_commands is not UNSET:
            field_dict["active_priv_commands"] = active_priv_commands
        if active_org_commands is not UNSET:
            field_dict["active_org_commands"] = active_org_commands
        if active_subcommands is not UNSET:
            field_dict["active_subcommands"] = active_subcommands
        if active_aliases is not UNSET:
            field_dict["active_aliases"] = active_aliases
        if active_events is not UNSET:
            field_dict["active_events"] = active_events
        if active_help_commands is not UNSET:
            field_dict["active_help_commands"] = active_help_commands

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        active_tell_commands = d.pop("active_tell_commands", UNSET)

        active_priv_commands = d.pop("active_priv_commands", UNSET)

        active_org_commands = d.pop("active_org_commands", UNSET)

        active_subcommands = d.pop("active_subcommands", UNSET)

        active_aliases = d.pop("active_aliases", UNSET)

        active_events = d.pop("active_events", UNSET)

        active_help_commands = d.pop("active_help_commands", UNSET)

        config_statistics = cls(
            active_tell_commands=active_tell_commands,
            active_priv_commands=active_priv_commands,
            active_org_commands=active_org_commands,
            active_subcommands=active_subcommands,
            active_aliases=active_aliases,
            active_events=active_events,
            active_help_commands=active_help_commands,
        )

        config_statistics.additional_properties = d
        return config_statistics

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
